// Part of SimCoupe - A SAM Coupe emulator
//
// UI.cpp: SDL user interface
//
//  Copyright (c) 1999-2006  Simon Owen
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Notes:
//  At present this module only really contains the event processing
//  code, for forwarding to other modules and processing fn keys

#include "SimCoupe.h"
#include "UI.h"

#include "Action.h"
#include "CPU.h"
#include "Display.h"
#include "Frame.h"
#include "GUI.h"
#include "Input.h"
#include "Options.h"

#include <SDL_Image.h>
#include <SDL_ttf.h>

#ifdef _DEBUG
#define WINDOW_CAPTION      "SimCoupe/SDL [DEBUG]"
#else
#define WINDOW_CAPTION      "SimCoupe/SDL"
#endif

#define CTRL_MODE_KEYPAD   2
#define CTRL_MODE_NUMPAD   3
#define CTRL_MODE_LETTERS  5
#define CTRL_MODE_ALL      (CTRL_MODE_KEYPAD * CTRL_MODE_NUMPAD * CTRL_MODE_LETTERS)

//HCF To save a zero in the cfg file, for compatibility purposes
int iMouseSpeed = 0;
////int iMouseSpeed = 3;
int iCtrlMode = CTRL_MODE_ALL;

//Configurable keys
int inCfgKey[4] = {-1, -1, -1, -1 };

extern SDL_Surface *picSlot;
extern SDL_Surface *picSlotElegido;
extern TTF_Font *font;
#define MAX_NOMBRE_FICHERO 128
extern char achConfigFile[MAX_NOMBRE_FICHERO];

//XBOX: KEYBOARD
#define KEYBOARD_ROWS 5
#define KEYBOARD_COLS 11
#define KEY_WIDTH   25   //22   /// 24   //48
#define KEY_HEIGHT 25  //50
#define KEY_DESPL_X  7    //8 /////15  //30
#define KEY_DESPL_Y  8   //15

//XBOX: MENU
#define MAX_SAVESTATES  9
//HCF Confirmation menus
#define OPTION_YESNO            0
#define DELAY_MENUS		       50
#define DELAY_MOVING_MENUS	  150
//Save confirmation
#define INICIO_CONFIRM_X    200
#define INICIO_CONFIRM_Y    200
#define CONFIRM_W           180
#define CONFIRM_H 		    100
#define DIALOGO_SEPARACION_CONFIRM_SLOTS  87
#define INICIO_CONFIRM_SLOT_X  215
#define INICIO_CONFIRM_SLOT_Y  233
#define TEXTO_CONFIRM_X  215
#define TEXTO_CONFIRM_Y  210
#define CONFIRM_DESPL_X  17
#define CONFIRM_DESPL_Y  17
#define MENU_RECTANGLE_OFFSET_Y 1
#define MENU_RECTANGLE_OFFSET_X 5
#define MENU_CURSOR_OFFSET_Y 2
#define MENU_CURSOR_OFFSET_X 17
#define MENU_FIRST_OPTION_Y   90
//#define MENU_FIRST_OPTION_Y   30
#define MENU_OPTION_HEIGHT    19    //18
#define MAX_MOUSE_SPEED  9

//HCF
int aainKeyboardCodes[KEYBOARD_ROWS][KEYBOARD_COLS] =
{
	SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7, SDLK_8, SDLK_9, SDLK_0, -1,
	SDLK_q, SDLK_w, SDLK_e, SDLK_r, SDLK_t, SDLK_y, SDLK_u, SDLK_i, SDLK_o, SDLK_p, SDLK_BACKSPACE, //DEL
	SDLK_a, SDLK_s, SDLK_d, SDLK_f, SDLK_g, SDLK_h, SDLK_j, SDLK_k, SDLK_l, SDLK_COLON, SDLK_SEMICOLON,   //: ,
	SDLK_TAB, SDLK_z, SDLK_x, SDLK_c, SDLK_v, SDLK_b, SDLK_n, SDLK_m, -1, SDLK_RETURN, SDLK_RETURN,   //TAB, ENTER
	SDLK_LCTRL, SDLK_LSHIFT, SDLK_SPACE, SDLK_SPACE, SDLK_SPACE, SDLK_SPACE, SDLK_SPACE, SDLK_SPACE, SDLK_RSHIFT, SDLK_RETURN, SDLK_RETURN  //SHIFT, CTRL, SHIFT, ENTER
};

//HCF
typedef struct
{
    int     nChar;                  // Symbol character (if any)

    eSamKey nSamKey, nSamModifiers; // Up to 2 SAM keys needed to generate the above symbol

    SDLKey  nKey;                   // SDL key
    int     nMods;                  // Modifier(s) to use with above key
}
COMBINATION_KEY;

typedef struct
{
    int     nChar;
    SDLKey  nKey;
}
SIMPLE_KEY;
extern SDL_Surface *pBack, *pFront;
extern SDL_Surface *picKeyboard;
extern SIMPLE_KEY asSamKeys [SK_MAX];
extern COMBINATION_KEY asSamSymbols[];
extern COMBINATION_KEY asSpectrumSymbols[];
bool UpdateKeyTable (SIMPLE_KEY* asKeys_, SDL_keysym* pKey_);
bool UpdateKeyTable (COMBINATION_KEY* asKeys_, SDL_keysym* pKey_);

bool g_fActive = true;

void vdReturnToDashboard();
void vdResetEmulator();

extern bool afKeyStates[SDLK_LAST], afKeys[SDLK_LAST];

//HCF Joystick
#define NUM_BOTONES 20

#define BOTON_AA 0
#define BOTON_BB 1
#define BOTON_XX 2
#define BOTON_YY 3

#define BOTON_ARRIBA 4
#define BOTON_ABAJO 5
#define BOTON_IZQUIERDA 6
#define BOTON_DERECHA 7

#define BOTON_BLANCO 8
#define BOTON_NEGRO 9
#define BOTON_START 10
#define BOTON_BACK 11
#define BOTON_LTRIGGER 12
#define BOTON_RTRIGGER 13
#define BOTON_LTHUMBSTICK 14
#define BOTON_RTHUMBSTICK 15
Sint16 blackbutton = 0;
Sint16 whitebutton = 0;
Sint16 abutton = 0; //A button
Sint16 bbutton = 0;
Sint16 xbutton = 0;
Sint16 ybutton = 0;
Sint16 backbutton = 0; //BACK button
Sint16 startbutton = 0; //START button
Sint16 rstick = 0; 
Sint16 lstick = 0; 
Sint16 dpad = 0; //dpad
Sint16 ltrigger, rtrigger = 0; //Trigger buttons
Sint16 joyx, joyy = 0; //axes
short ashBotones[NUM_BOTONES];
extern SDL_Joystick *pJoystick1;

void vdRellenaBotones(void)
{
	SDL_PumpEvents();
	SDL_JoystickUpdate(); //manual refresh of the gamepad(s)
	
	joyx = SDL_JoystickGetAxis(pJoystick1, 0);
	joyy = SDL_JoystickGetAxis(pJoystick1, 1);
	dpad = SDL_JoystickGetHat(pJoystick1, 0);
	blackbutton = SDL_JoystickGetButton(pJoystick1, 4); //Black Button
	whitebutton = SDL_JoystickGetButton(pJoystick1, 5); //White Button
	abutton = SDL_JoystickGetButton(pJoystick1, 0); //Get A-Button(0)
	bbutton = SDL_JoystickGetButton(pJoystick1, 1);
	backbutton = SDL_JoystickGetButton(pJoystick1, 9); //Get BACK-Button(9)
	lstick = SDL_JoystickGetButton(pJoystick1,10);
	rstick = SDL_JoystickGetButton(pJoystick1,11);
	ltrigger = SDL_JoystickGetButton(pJoystick1,6);
	rtrigger = SDL_JoystickGetButton(pJoystick1,7);
	xbutton = SDL_JoystickGetButton(pJoystick1,2);
	ybutton = SDL_JoystickGetButton(pJoystick1,3);
	startbutton = SDL_JoystickGetButton(pJoystick1,8);

	if(startbutton)
	{
	    ashBotones[BOTON_START] = 1;
	}
	else
	{
	    ashBotones[BOTON_START] = 0;
	}
	if(ltrigger)
	{
	    ashBotones[BOTON_LTRIGGER] = 1;
	}
	else
	{
	    ashBotones[BOTON_LTRIGGER] = 0;
	}
	if(rtrigger)
	{
	    ashBotones[BOTON_RTRIGGER] = 1;
	}
	else
	{
	    ashBotones[BOTON_RTRIGGER] = 0;
	}
	if(rstick)
	{
	    ashBotones[BOTON_RTHUMBSTICK] = 1;
	}
	else
	{
	    ashBotones[BOTON_RTHUMBSTICK] = 0;
	}
	if(backbutton)
	{
	    ashBotones[BOTON_BACK] = 1;
	}
	else
	{
	    ashBotones[BOTON_BACK] = 0;
	}
	if(abutton)
	{
	    ashBotones[BOTON_AA] = 1;
	}
	else
	{
	    ashBotones[BOTON_AA] = 0;
	}
	if(blackbutton)
	{
	    ashBotones[BOTON_NEGRO] = 1;
	}
	else
	{
	    ashBotones[BOTON_NEGRO] = 0;
	}
	if(whitebutton)
	{
	    ashBotones[BOTON_BLANCO] = 1;
	}
	else
	{
	    ashBotones[BOTON_BLANCO] = 0;
	}
	if(bbutton)
	{
	    ashBotones[BOTON_BB] = 1;
	}
	else
	{
	    ashBotones[BOTON_BB] = 0;
	}
	if(ybutton)
	{
	    ashBotones[BOTON_YY] = 1;
	}
	else
	{
	    ashBotones[BOTON_YY] = 0;
	}
	if(xbutton)
	{
	    ashBotones[BOTON_XX] = 1;
	}
	else
	{
	    ashBotones[BOTON_XX] = 0;
	}
	
	if(dpad & SDL_HAT_UP)
	{
		ashBotones[BOTON_ARRIBA] = 1;
	}
	else
	{
	    ashBotones[BOTON_ARRIBA] = 0;
	}
	if(dpad & SDL_HAT_DOWN)
	{
		ashBotones[BOTON_ABAJO] = 1;
	}
	else
	{
	    ashBotones[BOTON_ABAJO] = 0;
	}
	if(dpad & SDL_HAT_LEFT)
	{
		ashBotones[BOTON_IZQUIERDA] = 1;
	}
	else
	{
	    ashBotones[BOTON_IZQUIERDA] = 0;
	}
	if(dpad & SDL_HAT_RIGHT)
	{
		ashBotones[BOTON_DERECHA] = 1;
	}
	else
	{
	    ashBotones[BOTON_DERECHA] = 0;
	}
	
	if(lstick)
	{
	    ashBotones[BOTON_LTHUMBSTICK] = 1;
	}
	else
	{
	    ashBotones[BOTON_LTHUMBSTICK] = 0;
	}

}

//This function sets to zero all the keys of the Virtual Keyboard
void vdFillKeysToZero()
{
	int row, col;

	//First row
	for(col = 0; col < KEYBOARD_COLS - 1; col++)  //The last key is -1, so we don't update it
	{
		afKeys[aainKeyboardCodes[0][col]] = false;
	}

	//The rest of the rows
	for(row = 1; row < KEYBOARD_ROWS; row++)
	{
		for(col = 0; col < KEYBOARD_COLS; col++)
		{
			//if( aainKeyboardCodes[row][col] > 0 )
			afKeys[aainKeyboardCodes[row][col]] = false;
		}
	}
}

void vdRedrawScreen()
{
	//HCF WARNING: This may fail in other resolutions. Fix it!
	SDL_BlitSurface(pBack, NULL, pFront, NULL);
    SDL_Flip(pFront);
}

//inTypingMode -> 1 if typing mode, 0 if redefining keys
//inWhichKey -> Key to redefine
void vdVirtualKeyboard(int inTypingMode, int inWhichKey)
{

	char cAux = '0';
	int iRedibujar = 1;
	int salir = 0;
	int row = 0, col = 0;
	int movanterior = BOTON_AA;
	int iOpcionSeleccionada = 0;
	
	//SDL_Surface *SDLscreen;
	SDL_Surface *picSquare;
	SDL_Surface *picSquareTemp;
	SDL_Rect rectKeyboard;
	SDL_Rect rectSquare;
	
	/*
	int aainKeyboardCodes[KEYBOARD_ROWS][KEYBOARD_COLS] =
	{
		1, 2, 3, 4, 5, 6, 7, 8, 9, 10, -1,
		16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 14,   //DEL
		29, 30, 31, 32, 33, 34, 35, 36, 37, 49, 48,   //: ,
		15, 41, 42, 43, 44, 45, 46, 47, -1, 28, 28,   //TAB, ENTER
		112, 116, 52, 52, 52, 52, 52, 52, 112, 28, 28  //SHIFT, CTRL, SHIFT, ENTER
	};
	*/

	SDL_Color textColor = { 255, 255, 255 }; 

	//SDLscreen = SDL_GetVideoSurface();

	picSquareTemp = IMG_Load("d:\\square.png");	
	SDL_SetColorKey( picSquareTemp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(picSquareTemp->format, 255, 0, 255) );
	picSquare = SDL_DisplayFormatAlpha(picSquareTemp);
	SDL_FreeSurface(picSquareTemp);
	
	/*
	rectKeyboard.x = (SDLscreen->w - picKeyboard->w) / 2;
	rectKeyboard.y = (SDLscreen->h - picKeyboard->h) / 2;
	*/
	rectKeyboard.x = (pFront->w - picKeyboard->w) / 2;
	rectKeyboard.y = (pFront->h - picKeyboard->h) / 2;
	
	while(!salir)
	{
		vdRellenaBotones();
		if( dpad & SDL_HAT_UP )
		{
			if( movanterior != BOTON_ARRIBA )
			{
				if( row > 0 )
					row--;
				else
					row = KEYBOARD_ROWS - 1;

				iRedibujar = 1;
				
			}
			movanterior = BOTON_ARRIBA;
		}
		else if(movanterior == BOTON_ARRIBA)
		{
			movanterior = -1;
		}
		
		if( dpad & SDL_HAT_DOWN )
		{
			if( movanterior != BOTON_ABAJO )
			{
				if( row < KEYBOARD_ROWS - 1 )
					row++;
				else
					row = 0;

                iRedibujar = 1;
				
			}
			movanterior = BOTON_ABAJO;
		}
		else if(movanterior == BOTON_ABAJO)
		{
			movanterior = -1;
		}	
		if( dpad & SDL_HAT_LEFT )
		{
			if( movanterior != BOTON_IZQUIERDA )
			{
				if( col > 0 )
					col--;
				else
					col = KEYBOARD_COLS - 1;
                
				iRedibujar = 1;
								
			}
			movanterior = BOTON_IZQUIERDA;
		}
		else if(movanterior == BOTON_IZQUIERDA)
		{
			movanterior = -1;
		}
		
		if( dpad & SDL_HAT_RIGHT )
		{
			if( movanterior != BOTON_DERECHA )
			{
				if( col < KEYBOARD_COLS - 1 )
					col++;
				else
					col = 0;
					
				iRedibujar = 1;
				
			}
			movanterior = BOTON_DERECHA;
		}
		else if(movanterior == BOTON_DERECHA)
		{
			movanterior = -1;
		}	
		
		if( abutton )
		{
			if( movanterior != BOTON_AA )
			{
				if(inTypingMode)
				{
					//TYPE
					if( aainKeyboardCodes[row][col] > 0 )
					{
						afKeys[aainKeyboardCodes[row][col]] = true;
						/*
						keystat_senddata(aainKeyboardCodes[row][col]);
						keystat_senddata(aainKeyboardCodes[row][col] | 0x80);
						*/
					}
				}
				else
				{
					//CONFIGURE BUTTON (PENDING)
					/***/
					inCfgKey[inWhichKey] = aainKeyboardCodes[row][col];
					while(abutton)
						vdRellenaBotones();
					vdRedrawScreen();
					salir = 1;
					/***/
				}
			}
			movanterior = BOTON_AA;
		}
		else if(movanterior == BOTON_AA)
		{
			movanterior = -1;
		}
		
		if( bbutton )
		{
			//RESUME
			while(bbutton)
				vdRellenaBotones();
			
			// Blit the updated area and inform SDL it's changed
			/*
			SDL_BlitSurface(pBack, &rect, pFront, &rectFront);
			SDL_UpdateRects(pFront, 1, &rectFront);
			*/
			//scrndraw_draw(1);
            			
			salir = 1;
		}
			
		if( iRedibujar)
		{
			//Keyboard
			SDL_BlitSurface(picKeyboard, NULL, pFront, &rectKeyboard);
			//SDL_BlitSurface(picKeyboard, NULL, SDLscreen, &rectKeyboard);
						
			//RECTANGLE
			rectSquare.x = rectKeyboard.x + (KEY_WIDTH * col) + KEY_DESPL_X;
			rectSquare.y = rectKeyboard.y + (KEY_HEIGHT * row) + KEY_DESPL_Y;
			
			SDL_BlitSurface(picSquare, NULL, pFront, &rectSquare);
			SDL_Flip(pFront);	
			/*
			SDL_BlitSurface(picSquare, NULL, SDLscreen, &rectSquare);
			SDL_Flip(SDLscreen);	
			*/

		}
		
		iRedibujar = 0;
		SDL_Delay(20);
		
	}

	//HCF Redraw the screen of the game. 
	vdRedrawScreen();
	
	//HCF: Free the memory of the rectangle
	SDL_FreeSurface(picSquare);
}

//iOptions values:
//0->Create Blank Save
void vdDibujaConfirm(short shRet, char *achMensaje, int iOptions)
{
	int i;
		
	SDL_Rect rectDialogo;
	SDL_Rect rectNumero;
	SDL_Rect rectFrase;
	char achTexto[50];
	char achNumero[10];
	
	SDL_Surface *mainSurface;
	SDL_Surface *picTexto;

	SDL_Color textColor = { 255, 255, 255 }; 
	
	strcpy(achTexto, achMensaje);
						
	rectDialogo.x = INICIO_CONFIRM_X;
	rectDialogo.y = INICIO_CONFIRM_Y;
	rectDialogo.w = CONFIRM_W;
	rectDialogo.h = CONFIRM_H;

	//SDL_FillRect(screen, &rectDialogo, SDL_MapRGB(screen->format, 0, 128, 128));
	SDL_FillRect(pFront, &rectDialogo, SDL_MapRGB(pFront->format, 155, 80, 205));
	
	rectFrase.x = TEXTO_CONFIRM_X;
	rectFrase.y = TEXTO_CONFIRM_Y;
	
	picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
	SDL_BlitSurface(picTexto, NULL, pFront, &rectFrase);
	SDL_FreeSurface(picTexto);

	rectDialogo.y = INICIO_CONFIRM_SLOT_Y;

	//2 options
	for( i = 0; i < 2; i++ )
	{
		rectDialogo.x = ( i * DIALOGO_SEPARACION_CONFIRM_SLOTS ) + INICIO_CONFIRM_SLOT_X;
		SDL_BlitSurface(picSlot, NULL, pFront, &rectDialogo);
		
		if( i == 0 )
		{		
			if(iOptions == OPTION_YESNO)
			{
				strcpy(achNumero, "YES");
			}
		}
		else
		{
			if(iOptions == OPTION_YESNO)
			{
				strcpy(achNumero, "NO");
			}
		}
		
		picTexto = TTF_RenderText_Solid( font, achNumero, textColor ); 
		rectNumero.x = rectDialogo.x + CONFIRM_DESPL_X;
		rectNumero.y = rectDialogo.y + CONFIRM_DESPL_Y;

		SDL_BlitSurface(picTexto, NULL, pFront, &rectNumero);
		SDL_FreeSurface(picTexto);
		
	}
	
	//Choice square
	rectDialogo.y = INICIO_CONFIRM_SLOT_Y;
	if(shRet == 1)
		rectDialogo.x = INICIO_CONFIRM_SLOT_X;
	else
		rectDialogo.x = DIALOGO_SEPARACION_CONFIRM_SLOTS + INICIO_CONFIRM_SLOT_X;
	

	SDL_BlitSurface(picSlotElegido, NULL, pFront, &rectDialogo);
	SDL_Flip(pFront);
}

//iOptions values:
//0->Create Blank Save
short shConfirmWindow(char *achMensaje, int iOptions)
{
	short shReturn;
	short salir;
	short movanteriorH = BOTON_AA;
		
	shReturn = 1;
	vdDibujaConfirm(shReturn, achMensaje, iOptions);

	salir = 0;
	while(!salir)
	{
		//Se elige 0-NO o bien 1-SI
		vdRellenaBotones();	
		if( dpad & SDL_HAT_LEFT )
		{
			if( movanteriorH != BOTON_IZQUIERDA )
			{
				if( shReturn == 0 )
				{
					shReturn = 1;
					vdDibujaConfirm(shReturn, achMensaje, iOptions);
				}
			}
			movanteriorH = BOTON_IZQUIERDA;
		}
		else if(movanteriorH == BOTON_IZQUIERDA)
		{
			movanteriorH = -1;
		}	
		if( dpad & SDL_HAT_RIGHT )
		{
			if( movanteriorH != BOTON_DERECHA )
			{
				if( shReturn == 1 )
				{
					shReturn = 0;
					vdDibujaConfirm(shReturn, achMensaje, iOptions);
				}
			}
			movanteriorH = BOTON_DERECHA;
		}
		else if(movanteriorH == BOTON_DERECHA)
		{
			movanteriorH = -1;
		}
		
		if( abutton )
		{
			if( movanteriorH != BOTON_AA )
			{
				salir = 1;
			}
			movanteriorH = BOTON_AA;
		}
		else if(movanteriorH == BOTON_AA)
		{
			movanteriorH = -1;
		}
				
		Sleep(DELAY_MENUS);
	}
	
	return shReturn;
}

void vdDibujaOK(char *achMensaje)
{
	SDL_Rect rectDialogo;
	SDL_Rect rectNumero;
	SDL_Rect rectFrase;
	char achTexto[50];
	char achNumero[10];
	
	SDL_Surface *picTexto;
	SDL_Surface *mainSurface;
	
	SDL_Color textColor = { 255, 255, 255 }; 

	strcpy(achTexto, achMensaje);
						
	rectDialogo.x = INICIO_CONFIRM_X;
	rectDialogo.y = INICIO_CONFIRM_Y;
	rectDialogo.w = CONFIRM_W;
	rectDialogo.h = CONFIRM_H;

	//SDL_FillRect(screen, &rectDialogo, SDL_MapRGB(screen->format, 0, 128, 128));
	SDL_FillRect(pFront, &rectDialogo, SDL_MapRGB(pFront->format, 155, 80, 205));
	
	rectFrase.x = TEXTO_CONFIRM_X;
	rectFrase.y = TEXTO_CONFIRM_Y;
	
	picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
	SDL_BlitSurface(picTexto, NULL, pFront, &rectFrase);
	SDL_FreeSurface(picTexto);

	rectDialogo.y = INICIO_CONFIRM_SLOT_Y;
	rectDialogo.x = ( DIALOGO_SEPARACION_CONFIRM_SLOTS / 2 ) + INICIO_CONFIRM_SLOT_X;
	SDL_BlitSurface(picSlot, NULL, pFront, &rectDialogo);
	strcpy(achNumero, "OK");
		
	picTexto = TTF_RenderText_Solid( font, achNumero, textColor ); 
	rectNumero.x = rectDialogo.x + CONFIRM_DESPL_X;
	rectNumero.y = rectDialogo.y + CONFIRM_DESPL_Y;

	SDL_BlitSurface(picTexto, NULL, pFront, &rectNumero);
	SDL_FreeSurface(picTexto);
	
	//Choice square
	rectDialogo.y = INICIO_CONFIRM_SLOT_Y;
	rectDialogo.x = ( DIALOGO_SEPARACION_CONFIRM_SLOTS / 2 ) + INICIO_CONFIRM_SLOT_X;
	
	SDL_BlitSurface(picSlotElegido, NULL, pFront, &rectDialogo);
	SDL_Flip(pFront);
}

void vdLoadConfigFile(FILE *fdini)
{
	int inAux;
	
	fscanf(fdini, "%d\n", &inAux);
	inCfgKey[0] = inAux;

	fscanf(fdini, "%d\n", &inAux);
	inCfgKey[1] = inAux;

	fscanf(fdini, "%d\n", &inAux);
	inCfgKey[2] = inAux;

	fscanf(fdini, "%d\n", &inAux);
	inCfgKey[3] = inAux;

	fscanf(fdini, "%d\n", &inAux);
	iMouseSpeed = inAux;

	fscanf(fdini, "%d\n", &inAux);
	iCtrlMode = inAux;
}

void vdCreateConfigFile(char *achFileName)
{
	short shSalir;
	FILE *fdini;
	int pulsanterior = BOTON_AA;
	
	fdini = fopen(achFileName, "w+");
	if(fdini != NULL)
	{
		fprintf(fdini, "%d\n", inCfgKey[0]);
		fprintf(fdini, "%d\n", inCfgKey[1]);
		fprintf(fdini, "%d\n", inCfgKey[2]);
		fprintf(fdini, "%d\n", inCfgKey[3]);
		
		fprintf(fdini, "%d\n", iMouseSpeed);
		fprintf(fdini, "%d\n", iCtrlMode);
		
		fprintf(fdini, "0\n");  //RFU
		fprintf(fdini, "0\n");  //RFU
		fprintf(fdini, "0\n");  //RFU
		fprintf(fdini, "0\n");  //RFU

		fclose(fdini);
	}
		
	vdDibujaOK("INI created");
	shSalir = 0;
	while(!shSalir)
	{
		vdRellenaBotones();
		
		if( abutton )
		{
			if( pulsanterior != BOTON_AA )
			{
				shSalir = 1;	
			}
		}
		else if(pulsanterior == BOTON_AA)
		{
		     pulsanterior = -1;
		}

		Sleep(DELAY_MENUS);
	}

}

void vdSetCurrentText(char *achText, int inWhichKey)
{
	char achAux[50];
	
	switch( inCfgKey[inWhichKey] )
	{
		case 9:
			strcpy(achAux, "TAB");
			break;
		case 13:
			strcpy(achAux, "RETURN");
			break;
		case 306:
			strcpy(achAux, "LCTRL");
			break;
		case 304:
			strcpy(achAux, "LSHIFT");
			break;
		case 303:
			strcpy(achAux, "RSHIFT");
			break;
		case 32:
			strcpy(achAux, "SPACE");
			break;
		case 58:
			strcpy(achAux, ".");
			break;
		case 59:
			strcpy(achAux, ";");
			break;
		case 8:
			strcpy(achAux, "BACK");
			break;
		case -1:
			switch(inWhichKey)
			{
				case BOTON_AA:
					strcpy(achAux, "ENTER");
					break;
				case BOTON_BB:
					strcpy(achAux, "ESC");
					break;
				case BOTON_XX:
					strcpy(achAux, "SPACE");
					break;
				case BOTON_YY:
					strcpy(achAux, "0");
					break;
				default:
					strcpy(achAux, "None");
					break;
			}
			break;
		default:
			if( inCfgKey[inWhichKey] >= 48 && inCfgKey[inWhichKey] <= 57 )
			{
				achAux[0] = '0' + (inCfgKey[inWhichKey] - 48);
				achAux[1] = '\0';
			}
			else if( inCfgKey[inWhichKey] >= 97 && inCfgKey[inWhichKey] <= 122 )
			{
				achAux[0] = 'A' + (inCfgKey[inWhichKey] - 97);
				achAux[1] = '\0';
			}
			else
			{
				strcpy(achAux, "Unknown");
			}
			break;
	}
	strcpy(achText, "(current: ");
	strcat(achText, achAux);
	strcat(achText, ")");
}

void vdRedefineKeys()
{
	char cAux = '0';
	int iRedibujar = 1;
	//int iLongitud;
	int salir = 0;
	int movanterior = BOTON_AA;
	int iNumOpciones;
	int iOpcionSeleccionada = 0;

	int iVideoModeChanged = 0;
	
	//SDL_Surface *SDLscreen;
	SDL_Surface *picTexto;
	SDL_Surface *picRectangle;
	SDL_Surface *picRectangleTemp;
	SDL_Rect rectVentana;
	SDL_Rect rectTexto;
	
	char achTexto[150];
	char achCurrent[100];
	
	SDL_Color textColor = { 255, 255, 255 }; 
	SDL_Color textOrangeColor = { 255, 165, 0 }; 

	//SDLscreen = SDL_GetVideoSurface();

	picRectangleTemp = IMG_Load("d:\\rectangle.png");	
	SDL_SetColorKey( picRectangleTemp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(picRectangleTemp->format, 255, 0, 255) );
	picRectangle = SDL_DisplayFormatAlpha(picRectangleTemp);
	SDL_FreeSurface(picRectangleTemp);
	
	rectVentana.x = (pFront->w - 250) / 2;
	rectVentana.y = 50;
	rectVentana.w = 250;
	//rectVentana.h = 160;
	rectVentana.h = 190;

	rectTexto.x = rectVentana.x + 30;
	rectTexto.y = 30;
	
	//iNumOpciones = 5;	
	iNumOpciones = 6;	

	while(!salir)
	{
		vdRellenaBotones();
		if( dpad & SDL_HAT_UP )
		{
			if( movanterior != BOTON_ARRIBA )
			{
				if( iOpcionSeleccionada > 0 )
					iOpcionSeleccionada--;
				else
					iOpcionSeleccionada = iNumOpciones - 1;

				iRedibujar = 1;
			}
			movanterior = BOTON_ARRIBA;
		}
		else if(movanterior == BOTON_ARRIBA)
		{
			movanterior = -1;
		}
		
		if( dpad & SDL_HAT_DOWN )
		{
			if( movanterior != BOTON_ABAJO )
			{
				if( iOpcionSeleccionada < iNumOpciones - 1 )
					iOpcionSeleccionada++;
				else
					iOpcionSeleccionada = 0;

				iRedibujar = 1;
			}
			movanterior = BOTON_ABAJO;
		}
		else if(movanterior == BOTON_ABAJO)
		{
			movanterior = -1;
		}	
		
		if( abutton )
		{
			if( movanterior != BOTON_AA )
			{
				if( iOpcionSeleccionada == 0 )
				{
					vdVirtualKeyboard(0, BOTON_AA);
					iRedibujar = 1;
				}
				else if( iOpcionSeleccionada == 1 )
				{
					vdVirtualKeyboard(0, BOTON_BB);
					iRedibujar = 1;
				}
				else if( iOpcionSeleccionada == 2 )
				{
					vdVirtualKeyboard(0, BOTON_XX);
					iRedibujar = 1;

				}
				else if( iOpcionSeleccionada == 3 )
				{
					vdVirtualKeyboard(0, BOTON_YY);
					iRedibujar = 1;

				}
				else if( iOpcionSeleccionada == 4 )
				{
					switch(iCtrlMode)
					{
						case CTRL_MODE_ALL:
							iCtrlMode = CTRL_MODE_NUMPAD;
						break;
						case CTRL_MODE_NUMPAD:
							iCtrlMode = CTRL_MODE_KEYPAD;
						break;
						case CTRL_MODE_KEYPAD:
							iCtrlMode = CTRL_MODE_LETTERS;
						break;
						case CTRL_MODE_LETTERS:
							iCtrlMode = CTRL_MODE_ALL;
						break;
						default:
							iCtrlMode = CTRL_MODE_ALL;
						break;

					}
					iRedibujar = 1;

				}
				else if( iOpcionSeleccionada == 5 )
				{
					while(abutton)
						vdRellenaBotones();

					salir = 1;
				}
				
			}
			movanterior = BOTON_AA;
		}
		else if(movanterior == BOTON_AA)
		{
			movanterior = -1;
		}	
		
		if( bbutton )
		{
			//RESUME
			while(bbutton)
				vdRellenaBotones();

			salir = 1;
		}
			
		if( iRedibujar)
		{
			rectVentana.x = (pFront->w - 250) / 2;
			rectVentana.y = 50;
			rectVentana.h = 190;
			
			//HCF This menu has a different color
			SDL_FillRect(pFront, &rectVentana, SDL_MapRGB(pFront->format, 155, 80, 205));
			//SDL_FillRect(pFront, &rectVentana, SDL_MapRGB(pFront->format, 0, 50, 50));
			
			//TEXT AA
			strcpy(achTexto, "Redef A button ");
			vdSetCurrentText(achCurrent, BOTON_AA);
			strcat(achTexto, achCurrent);
			rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
		
			//TEXT BB
			strcpy(achTexto, "Redef B button ");
			vdSetCurrentText(achCurrent, BOTON_BB);
			strcat(achTexto, achCurrent);
			rectTexto.y = MENU_FIRST_OPTION_Y;
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);

			//TEXT XX
			strcpy(achTexto, "Redef X button ");
			vdSetCurrentText(achCurrent, BOTON_XX);
			strcat(achTexto, achCurrent);
			rectTexto.y = MENU_FIRST_OPTION_Y + MENU_OPTION_HEIGHT;
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//TEXT YY
			strcpy(achTexto, "Redef Y button ");
			vdSetCurrentText(achCurrent, BOTON_YY);
			strcat(achTexto, achCurrent);
			rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);

			//TEXT YY
			strcpy(achTexto, "D-pad ");
			strcat(achTexto, "(current: ");
			
			//vdSetCurrentText(achCurrent, BOTON_YY);
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					strcat(achTexto, "All");
				break;
				case CTRL_MODE_NUMPAD:
					strcat(achTexto, "Numpad");
				break;
				case CTRL_MODE_KEYPAD:
					strcat(achTexto, "Keypad");
				break;
				case CTRL_MODE_LETTERS:
					strcat(achTexto, "Letters");
				break;
				default:
					strcat(achTexto, "Unknown");
				break;

			}
			
			strcat(achTexto, ")");

			rectTexto.y = ( (3 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
	
			//TEXT BACK
			strcpy(achTexto, "Back");
			rectTexto.y = ( (4 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//TEXT INSTRUCTIONS
			strcpy(achTexto, "A: Select   B: Cancel ");
			rectTexto.y = 200;

			picTexto = TTF_RenderText_Solid( font, achTexto, textOrangeColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//RECTANGLE
			switch(iOpcionSeleccionada)
			{
				case 0:
					rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
					break;
				case 1:
					rectTexto.y = MENU_FIRST_OPTION_Y;
					break;
				case 2:
					rectTexto.y = MENU_FIRST_OPTION_Y + MENU_OPTION_HEIGHT;
					break;
				case 3:
					rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 4:
					rectTexto.y = ( (3 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 5:
					rectTexto.y = ( (4 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 6:
					rectTexto.y = ( (5 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 7:
					rectTexto.y = ( (6 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 8:
					rectTexto.y = ( (7 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				default:
					rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
					break;	
			}
			
			rectTexto.y -= MENU_RECTANGLE_OFFSET_Y;
			rectTexto.x -= MENU_RECTANGLE_OFFSET_X;
			SDL_BlitSurface(picRectangle, NULL, pFront, &rectTexto);
			rectTexto.x += MENU_RECTANGLE_OFFSET_X;
						
			SDL_Flip(pFront);	
		}
		
		iRedibujar = 0;
		SDL_Delay(20);
		
	}

	//HCF: Free the memory of the rectangle
	SDL_FreeSurface(picRectangle);
}

void vdMainMenu(void)
{
	char cAux = '0';
		
	int iRedibujar = 1;
	//int iLongitud;
	int salir = 0;
	int movanterior = BOTON_AA;
	int iNumOpciones;
	int iOpcionSeleccionada = 0;

	int iVideoModeChanged = 0;
	
	SDL_Surface *SDLscreen;
	SDL_Surface *picTexto;
	SDL_Surface *picRectangle;
	SDL_Surface *picRectangleTemp;
	SDL_Rect rectVentana;
	SDL_Rect rectTexto;
	
	char achTexto[100];
	
	SDL_Color textColor = { 255, 255, 255 }; 
	SDL_Color textOrangeColor = { 255, 165, 0 }; 

	SDLscreen = SDL_GetVideoSurface();

	picRectangleTemp = IMG_Load("d:\\rectangle.png");	
	SDL_SetColorKey( picRectangleTemp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(picRectangleTemp->format, 255, 0, 255) );
	picRectangle = SDL_DisplayFormatAlpha(picRectangleTemp);
	SDL_FreeSurface(picRectangleTemp);
	
	rectVentana.x = (SDLscreen->w - 250) / 2;
	
	//rectVentana.y = 0;
	rectVentana.y = 50;
		
	rectVentana.w = 250;
	//rectVentana.h = 270; //260; 
	rectVentana.h = 180;   //210;

	rectTexto.x = rectVentana.x + 30;
	//rectTexto.y = 30;
	rectTexto.y = 70;
	
	iNumOpciones = 5;   //6;   //8;

	while(!salir)
	{
		vdRellenaBotones();
		if( dpad & SDL_HAT_UP )
		{
			if( movanterior != BOTON_ARRIBA )
			{
				if( iOpcionSeleccionada > 0 )
					iOpcionSeleccionada--;
				else
					iOpcionSeleccionada = iNumOpciones - 1;

				iRedibujar = 1;
				
			}
			movanterior = BOTON_ARRIBA;
		}
		else if(movanterior == BOTON_ARRIBA)
		{
			movanterior = -1;
		}
		
		if( dpad & SDL_HAT_DOWN )
		{
			if( movanterior != BOTON_ABAJO )
			{
				if( iOpcionSeleccionada < iNumOpciones - 1 )
					iOpcionSeleccionada++;
				else
					iOpcionSeleccionada = 0;

				iRedibujar = 1;

			}
			movanterior = BOTON_ABAJO;
		}
		else if(movanterior == BOTON_ABAJO)
		{
			movanterior = -1;
		}	
		
		if( abutton )
		{
			if( movanterior != BOTON_AA )
			{
				if( iOpcionSeleccionada == 0 )
				{
					//RESUME
					vdRedrawScreen();
					while(abutton)
						vdRellenaBotones();
					salir = 1;
				}
				else if( iOpcionSeleccionada == 1 )
				{
					//RESET
					if( shConfirmWindow("Reset?", OPTION_YESNO) )
					{
						vdResetEmulator();
					}
					vdRedrawScreen();
					iRedibujar = 1;
					
					//HCF It will probably need an F12 press too!
					//afKeys[265] = true;  //BOOT
					//salir = 1;
				}
				else if( iOpcionSeleccionada == 2 )
				{
					//REDEFINE KEYS
					vdRedrawScreen();
					vdRedefineKeys();
					vdRedrawScreen();
					
					//Back to the main menu after redefining keys
					//salir = 1;
					iRedibujar = 1;

				}
				/*
				else if( iOpcionSeleccionada == 3 )
				{
					//MOUSE SPEED
					if(iMouseSpeed < MAX_MOUSE_SPEED)
						iMouseSpeed++;
					else
						iMouseSpeed = 1;
					
					iRedibujar = 1;
				}
				*/
				else if( iOpcionSeleccionada == 3 )
				{
					//SAVE CONFIGURATION AS INI FILE
					if( shConfirmWindow("Save Config?", OPTION_YESNO) )
					{
						vdCreateConfigFile(achConfigFile);
					}
					vdRedrawScreen();
					iRedibujar = 1;
				}
				else if( iOpcionSeleccionada == 4 )
				{
					//EXIT TO ROM LIST
					if( shConfirmWindow("Exit?", OPTION_YESNO) )
					{
						vdReturnToDashboard();
					}
					vdRedrawScreen();
					iRedibujar = 1;
				}
			}
			movanterior = BOTON_AA;
		}
		else if(movanterior == BOTON_AA)
		{
			movanterior = -1;
		}	
		
		if( bbutton )
		{
			//RESUME
			while(bbutton)
				vdRellenaBotones();
			vdRedrawScreen();
			salir = 1;
		}
			
		if( iRedibujar)
		{
			rectVentana.x = (SDLscreen->w - 250) / 2;
			//rectVentana.y = 0;
			rectVentana.y = 50;
			rectVentana.h = 180;   //210;    //240;   //260;
			
			SDL_FillRect(SDLscreen, &rectVentana, SDL_MapRGB(SDLscreen->format, 0, 50, 50));

			//TEXT RESUME
			strcpy(achTexto, "Resume");
			rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
		
			//TEXT RESET
			strcpy(achTexto, "Reset");
			rectTexto.y = MENU_FIRST_OPTION_Y;
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);

			/*
			//TEXT DISK IN FLOPPY A
			strcpy(achTexto, "Disk in Floppy A");
			rectTexto.y = ( (1 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);

			//TEXT DISK IN FLOPPY B
			strcpy(achTexto, "Disk in Floppy B");
			rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			*/

			//TEXT REDEF KEYS
			strcpy(achTexto, "Redefine keys");
			rectTexto.y = ( (1 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//TEXT Mouse speed
			/*
			sprintf(achTexto, "Mouse speed: %d", iMouseSpeed);
			rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			*/
			
			//TEXT Save config as INI file
			strcpy(achTexto, "Save config as INI file");
			rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);

			//TEXT EXIT TO ROM LIST
			strcpy(achTexto, "Exit to rom list");
			rectTexto.y = ( (3 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
			picTexto = TTF_RenderText_Solid( font, achTexto, textColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//TEXT INSTRUCTIONS
			strcpy(achTexto, "A: Select   B: Cancel ");
			rectTexto.y = 190;   //210;  

			picTexto = TTF_RenderText_Solid( font, achTexto, textOrangeColor ); 
			SDL_BlitSurface(picTexto, NULL, pFront, &rectTexto);
			SDL_FreeSurface(picTexto);
			
			//RECTANGLE
			switch(iOpcionSeleccionada)
			{
				case 0:
					rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
					break;
				case 1:
					rectTexto.y = MENU_FIRST_OPTION_Y;
					break;
				case 2:
					rectTexto.y = MENU_FIRST_OPTION_Y + MENU_OPTION_HEIGHT;
					break;
				case 3:
					rectTexto.y = ( (2 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 4:
					rectTexto.y = ( (3 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 5:
					rectTexto.y = ( (4 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 6:
					rectTexto.y = ( (5 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 7:
					rectTexto.y = ( (6 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 8:
					rectTexto.y = ( (7 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 9:
					rectTexto.y = ( (8 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 10:
					rectTexto.y = ( (9 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 11:
					rectTexto.y = ( (10 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				case 12:
					rectTexto.y = ( (11 * MENU_OPTION_HEIGHT) + MENU_FIRST_OPTION_Y );
					break;
				default:
					rectTexto.y = MENU_FIRST_OPTION_Y - MENU_OPTION_HEIGHT;
					break;	
			}
			
			rectTexto.y -= MENU_RECTANGLE_OFFSET_Y;
			rectTexto.x -= MENU_RECTANGLE_OFFSET_X;
			SDL_BlitSurface(picRectangle, NULL, pFront, &rectTexto);
			rectTexto.x += MENU_RECTANGLE_OFFSET_X;
						
			SDL_Flip(pFront);	
		}
		
		iRedibujar = 0;
		SDL_Delay(20);
		
	}

	//HCF: Free the memory of the rectangle
	SDL_FreeSurface(picRectangle);
	
}

bool UI::Init (bool fFirstInit_/*=false*/)
{
    bool fRet = true;

    Exit(true);
    TRACE("-> UI::Init()\n");

    // Set the window caption and disable the cursor until needed
    //SDL_WM_SetCaption(WINDOW_CAPTION, WINDOW_CAPTION);
    SDL_ShowCursor(SDL_DISABLE);

	//HCF We don't need this window
	/**
    // To help on platforms without a native GUI, we'll display a one-time welcome message
#if !defined(__APPLE__) && !defined(_WINDOWS)
    if (GetOption(firstrun))
    {
        // Clear the option so we don't show it again
        SetOption(firstrun, 0);

        // Simple message box showing some keys
        GUI::Start(new CMessageBox(NULL,
                    "Some useful keys to get you started:\n\n"
                    "  F1 - Insert disk image\n"
                    "  F10 - Options\n"
                    "  F12 - Reset\n"
                    "  Ctrl-F12 - Exit emulator\n"
                    "  Numpad-9 - Boot drive 1\n\n"
                    "Consult the README for further details.",
                    "Welcome to SimCoupe!",
                    mbInformation));
    }
#endif
	**/

    TRACE("<- UI::Init() returning %s\n", fRet ? "true" : "false");

    return fRet;
}

void UI::Exit (bool fReInit_/*=false*/)
{
    TRACE("-> UI::Exit(%s)\n", fReInit_ ? "reinit" : "");

    if (!fReInit_)
        SDL_QuitSubSystem(SDL_INIT_TIMER);

    TRACE("<- UI::Exit()\n");
}


// Check and process any incoming messages
bool UI::CheckEvents ()
{
	SDL_keysym* pKey;
    SDL_Event event;

    // If the GUI is active the input won't be polled by CPU.cpp, so do it here
    if (GUI::IsActive())
        Input::Update();

    // Re-pause after a single frame-step
    if (g_fFrameStep)
        Action::Do(actFrameStep);


	//HCF Initialize every key to NOT pressed
	//memcpy(afKeys, 0x00, 323 * sizeof(bool) );

	vdRellenaBotones();

	//Assign 0 to the keys of the virtual keyboard
	vdFillKeysToZero();

	if(ashBotones[BOTON_START] && ashBotones[BOTON_BACK])
	{
		//RETURN TO DASHBOARD
		vdReturnToDashboard();
	}
	else if(ashBotones[BOTON_START])
	{
		//MAIN MENU
		SDL_PauseAudio(1);
		vdMainMenu();
		SDL_PauseAudio(0);
	}
	else if(ashBotones[BOTON_LTRIGGER] && ashBotones[BOTON_RTRIGGER])
	{
		//TYPING MODE
		SDL_PauseAudio(1);
		vdVirtualKeyboard(1, 0);
		SDL_PauseAudio(0);
	}
	else
	{
		//HCF: Directions (Keypad + OPQA)
		if(ashBotones[BOTON_ABAJO])
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_DOWN] = true;
					afKeys[SDLK_KP2] = true;
					afKeys[SDLK_a] = true;
					asSamKeys[8].nKey = SDLK_a;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP2] = true;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_DOWN] = true;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_a] = true;
					asSamKeys[8].nKey = SDLK_a;
				break;
			}
			
		}
		else
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_DOWN] = false;
					afKeys[SDLK_KP2] = false;
					afKeys[SDLK_a] = false;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP2] = false;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_DOWN] = false;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_a] = false;
				break;
			}
			
		}
		if(ashBotones[BOTON_ARRIBA])
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_UP] = true;
					afKeys[SDLK_KP8] = true;
					afKeys[SDLK_q] = true;
					asSamKeys[16].nKey = SDLK_q;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP8] = true;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_UP] = true;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_q] = true;
					asSamKeys[16].nKey = SDLK_q;
				break;
			}
			
		}
		else
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_UP] = false;
					afKeys[SDLK_KP8] = false;
					afKeys[SDLK_q] = false;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP8] = false;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_UP] = false;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_q] = false;
				break;
			}
			
		}
		if(ashBotones[BOTON_DERECHA])
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_RIGHT] = true;
					afKeys[SDLK_KP6] = true;
					afKeys[SDLK_p] = true;
					asSamKeys[40].nKey = SDLK_p;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP6] = true;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_RIGHT] = true;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_p] = true;
					asSamKeys[40].nKey = SDLK_p;
				break;
			}
			
		}
		else
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_RIGHT] = false;
					afKeys[SDLK_KP6] = false;
					afKeys[SDLK_p] = false;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP6] = false;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_RIGHT] = false;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_p] = false;
				break;
			}
			
		}
		if(ashBotones[BOTON_IZQUIERDA])
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_LEFT] = true;
					afKeys[SDLK_KP4] = true;
					afKeys[SDLK_o] = true;
					asSamKeys[41].nKey = SDLK_o;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP4] = true;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_LEFT] = true;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_o] = true;
					asSamKeys[41].nKey = SDLK_o;
				break;
			}
			
		}
		else
		{
			switch(iCtrlMode)
			{
				case CTRL_MODE_ALL:
					afKeys[SDLK_LEFT] = false;
					afKeys[SDLK_KP4] = false;
					afKeys[SDLK_o] = false;
				break;
				case CTRL_MODE_NUMPAD:
					afKeys[SDLK_KP4] = false;
				break;
				case CTRL_MODE_KEYPAD:
					afKeys[SDLK_LEFT] = false;
					break;
				case CTRL_MODE_LETTERS:
					afKeys[SDLK_o] = false;
				break;
			}
		}

		//HCF: ACTIONS
		if(ashBotones[BOTON_AA])
		{
			if(inCfgKey[BOTON_AA] < 0)
				afKeys[SDLK_RETURN] = true;
			else
				afKeys[inCfgKey[BOTON_AA]] = true;

		}
		else
		{
			if(inCfgKey[BOTON_AA] < 0)
				afKeys[SDLK_RETURN] = false;
			else
				afKeys[inCfgKey[BOTON_AA]] = false;
		}

		if(ashBotones[BOTON_BB])
		{
			if(inCfgKey[BOTON_BB] < 0)
				afKeys[SDLK_ESCAPE] = true;
			else
				afKeys[inCfgKey[BOTON_BB]] = true;
			
		}
		else
		{
			if(inCfgKey[BOTON_BB] < 0)
				afKeys[SDLK_ESCAPE] = false;
			else
				afKeys[inCfgKey[BOTON_BB]] = false;

		}
		
		if(ashBotones[BOTON_XX])
		{
			/*
			afKeys[SDLK_SPACE] = true;
			asSamKeys[56].nKey = SDLK_SPACE;
			*/
			if(inCfgKey[BOTON_XX] < 0)
			{
				afKeys[SDLK_SPACE] = true;
				asSamKeys[56].nKey = SDLK_SPACE;
			}
			else
			{
				afKeys[inCfgKey[BOTON_XX]] = true;
				//HCF Fix this!!!
				//asSamKeys[56].nKey = inCfgKey[BOTON_XX];
			}
		}
		else
		{
			if(inCfgKey[BOTON_XX] < 0)
				afKeys[SDLK_SPACE] = false;
			else
				afKeys[inCfgKey[BOTON_XX]] = false;
		}

		if(ashBotones[BOTON_YY])
		{
			if(inCfgKey[BOTON_YY] < 0)
				afKeys[SDLK_KP0] = true;
			else
				afKeys[inCfgKey[BOTON_YY]] = true;
		}
		else
		{
			//afKeys[SDLK_KP0] = false;

			if(inCfgKey[BOTON_YY] < 0)
				afKeys[SDLK_KP0] = false;
			else
				afKeys[inCfgKey[BOTON_YY]] = false;
		}

	}
    return true;
}

void UI::ShowMessage (eMsgType eType_, const char* pcszMessage_)
{
    if (eType_ == msgInfo)
        GUI::Start(new CMessageBox(NULL, pcszMessage_, WINDOW_CAPTION, mbInformation));
    else if (eType_ == msgWarning)
        GUI::Start(new CMessageBox(NULL, pcszMessage_, WINDOW_CAPTION, mbWarning));
    else
        GUI::Start(new CMessageBox(NULL, pcszMessage_, WINDOW_CAPTION, mbError));
}

////////////////////////////////////////////////////////////////////////////////

bool UI::DoAction (int nAction_, bool fPressed_)
{
    // Key pressed?
    if (fPressed_)
    {
        switch (nAction_)
        {
            case actChangeWindowSize:
                GUI::Start(new CMessageBox(NULL, "Window sizing is not supported under SDL", "Sorry!", mbInformation));
                break;

            case actExitApplication:
            {
                SDL_Event event = { SDL_QUIT };
                SDL_PushEvent(&event);
                break;
            }

            case actPause:
            {
                // Reverse logic because we've the default processing hasn't occurred yet
                if (g_fPaused)
                    SDL_WM_SetCaption(WINDOW_CAPTION, WINDOW_CAPTION);
                else
                    SDL_WM_SetCaption(WINDOW_CAPTION " - Paused", WINDOW_CAPTION " - Paused");

                // Perform default processing
                return false;
            }

            // Perform the switch on key-up, to avoid an SDL 1.2.x bug
            case actToggleFullscreen:
                break;

            case actToggle5_4:
#ifndef USE_OPENGL
                GUI::Start(new CMessageBox(NULL, "5:4 mode is not available under SDL", "Sorry!", mbInformation));
#endif
                break;
            // Not processed
            default:
                return false;
        }
    }
    else    // Key released
    {
        switch (nAction_)
        {
            // To avoid an SDL bug (in 1.2.0 anyway), we'll do the following on key up instead of down
            case actToggleFullscreen:
                SetOption(fullscreen, !GetOption(fullscreen));
                Sound::Silence();
                Frame::Init();

                // Grab the mouse automatically in full-screen, or release in windowed mode
                Input::Acquire(!!GetOption(fullscreen), !GUI::IsActive());
                break;

#ifdef USE_OPENGL
            case actToggle5_4:
                SetOption(ratio5_4, !GetOption(ratio5_4));
                Frame::Init();
                Frame::SetStatus("%s aspect ratio", GetOption(ratio5_4) ? "5:4" : "1:1");
                break;
#endif
            // Not processed
            default:
                return false;
        }
    }

    // Action processed
    return true;
}
